@@ -1,15 +1,19 @@
-use shakmaty::Piece;
-use shakmaty::{
-    attacks, fen::Fen, Bitboard, Board, CastlingMode, Chess, Color, Color::Black, Color::White,
-    Rank, Role, Square,
+use std::{
+    cmp::Ordering,
+    fmt,
+    hash::{Hash, Hasher},
 };
-use std::fmt;
-use std::hash::{Hash, Hasher};
 
-use std::cmp::Ordering;
+use shakmaty::{
+    attacks,
+    fen::Fen,
+    Bitboard, Board, CastlingMode, Chess, Color,
+    Color::{Black, White},
+    FromSetup, Piece, PositionError, Rank, Role, Setup, Square,
+};
 
 use crate::{
-    MoveKind::EnPassant, MoveKind::Normal, MoveKind::UnPromotion, MoveKind::Uncapture,
+    MoveKind::{EnPassant, Normal, UnPromotion, Uncapture},
     RetroPockets, UnMove, UnMoveList,
 };
 
@@ -262,12 +266,14 @@ impl RetroBoard {
     }
 
     #[inline]
+    #[must_use]
     fn occupied(&self) -> Bitboard {
         self.board.occupied()
     }
 
     #[inline]
-    fn king_of(&self, color: Color) -> Square {
+    #[must_use]
+    pub fn king_of(&self, color: Color) -> Square {
         self.board.king_of(color).unwrap()
     }
 
@@ -436,6 +442,19 @@ impl fmt::Debug for RetroBoard {
     }
 }
 
+impl FromSetup for RetroBoard {
+    /// [`RetroPocket`](crate::RetroPocket) will be empty for both colors
+    fn from_setup(setup: &dyn Setup, _: CastlingMode) -> Result<Self, PositionError<Self>> {
+        Ok(Self {
+            board: setup.board().clone(),
+            retro_turn: !setup.turn(),
+            ep_square: setup.ep_square(),
+            halfmoves: 0,
+            pockets: RetroPockets::default(),
+        })
+    }
+}
+
 /// Consider valid positions with too many/impossible checkers (unreachable positions)
 impl From<RetroBoard> for Chess {
     fn from(item: RetroBoard) -> Self {
@@ -501,12 +520,14 @@ fn closest_further_square(bb: Bitboard, of: Square) -> (Square, Square) {
 
 #[cfg(test)]
 mod tests {
-    use super::*;
+    // use pretty_assertions::{assert_eq, assert_ne};
+    use std::collections::HashSet;
+
     use indoc::indoc;
     use paste::paste;
     use shakmaty::{uci::Uci, Position};
-    // use pretty_assertions::{assert_eq, assert_ne};
-    use std::collections::HashSet;
+
+    use super::*;
 
     fn u(s: &str) -> UnMove {
         UnMove::from_retro_uci(s).unwrap()
@@ -543,10 +564,10 @@ mod tests {
     }
 
     #[test]
-    fn new_no_pockets() {
+    fn test_new_no_pockets() {
         let r =
             RetroBoard::new_no_pockets("rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq - 0 1")
-                .expect("Retroboard bc fen is legal");
+                .expect("Retroboard because fen is legal");
         assert_eq!(r.retro_turn, Black);
         assert_eq!(
             r.board,
@@ -557,6 +578,18 @@ mod tests {
         assert_eq!(r.halfmoves, 0);
     }
 
+    #[test]
+    fn test_from_setup() {
+        let r =
+            RetroBoard::new_no_pockets("rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq - 0 1")
+                .expect("Retroboard because fen is legal");
+        let fen: Fen = "rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq - 0 1"
+            .parse()
+            .unwrap();
+        let r_setup = RetroBoard::from_setup(&fen, CastlingMode::Standard).unwrap();
+        assert_eq!(r, r_setup);
+    }
+
     #[test]
     fn test_hash() {
         let mut r =
